﻿// (c) IMT - Information Management Technology AG, CH-9470 Buchs, www.imt.ch. 
// SW guideline: SOP-Coding Guidelines Ver. 1.6
// Autor: Willener

using imtanalytics.ImtUsbProtocol;
using imtanalytics.ImtUsbProtocol.Device;
using imtanalytics.ImtUsbProtocol.Device.Citrex.RemoteObjects;
using imtanalytics.ImtUsbProtocol.Device.PF300Pro.RemoteObjects;
using imtanalytics.ImtUsbProtocol.EnumIdentifiers;
using ImtUsbProtocol.Citrex;
using ImtUsbProtocol.PF300Pro;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Management;
using System.Reflection;
using System.Threading;
using System.Windows;
using System.Linq;
using System.IO.Ports;
using System.Diagnostics;
using System.Text.RegularExpressions;
using ImtUsbProtcolTestApp.Device;
using imtanalytics.ImtUsbProtocol.Debug;

namespace ImtUsbProtcolTestApp {

    /// <summary>
    /// Test application for PF300Pro and the Citrex H5 Assembly "imtanalytics.ImtUsbProtocol" settings.
    /// </summary>
    public partial class MainWindow : Window {

        /// <summary>
        /// Receive the PF300Pro TriggerSettingsRemoteObjects.
        /// </summary>
        public EventHandler<TriggerSettingsRemoteObject>? TriggerSettingsRemoteObjectReceiveEvent;

        /// <summary>
        /// Receive the PF300Pro  DeviceSettingsRemoteObjects.
        /// </summary>
        public EventHandler<DeviceSettingsRemoteObject>? DeviceSettingsReceivedReceiveEvent;

        /// <summary>
        /// Receive the PF300Pro  MeasurementSettingsRemoteObject Event
        /// </summary>
        public EventHandler<MeasurementSettingsRemoteObject>? MeasurementSettingsRemoteObjectReceiveEvent;

        /// <summary>
        /// Receive the Citrex H5 DeviceConfigRemoteObjects.
        /// </summary>
        public EventHandler<DeviceConfigRemoteObject>? DeviceConfigRemoteObjectReceiveEvent;

        /// <summary>
        /// Receive the Citrex H5 TriggerConfigRemoteObjects.
        /// </summary>
        public EventHandler<TriggerConfigRemoteObject>? TriggerConfigRemoteObjectReceiveEvent;

        /// <summary>
        /// Constructor
        /// </summary>
        public MainWindow() {
            AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(CurrentDomain_UnhandledException);

            InitializeComponent();

            // Debug output from DLL, remove if not needed.
            DebugOutput.DebugLevel = DebugLevel.Standard;
            DebugOutput.SetDebugOutput();

            m_content = new DeviceSettings();
            this.DataContext = m_content;
            m_SettingsReceived = false;
            try {
                PF300Pro.Visibility = Visibility.Visible;
                Citrex_H5.Visibility = Visibility.Visible;
                Devices.SelectedIndex = 0;

                DeviceConfigRemoteObjectReceiveEvent += OnDeviceConfigRemoteObjectReceiveEvent;
                TriggerSettingsRemoteObjectReceiveEvent += OnTriggerSettingsRemoteObjectReceiveEvent;
                DeviceSettingsReceivedReceiveEvent += OnDeviceSettingsReceiveEvent;
                MeasurementSettingsRemoteObjectReceiveEvent += OnMeasurementSettingsRemoteObjectReceiveEvent;
                TriggerConfigRemoteObjectReceiveEvent += OnTriggerConfigRemoteObjectEventReceiveEvent;
            }
            catch (Exception ex) {
                Debug.WriteLine(ex.Message);
                MessageBox.Show("Error: Lost connection: " + ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error, MessageBoxResult.OK);
                Disconnect(this, new RoutedEventArgs());
            }
        }


        //On closing the application.
        protected override void OnClosing(CancelEventArgs e) {
            TriggerSettingsRemoteObjectReceiveEvent -= OnTriggerSettingsRemoteObjectReceiveEvent;
            DeviceSettingsReceivedReceiveEvent -= OnDeviceSettingsReceiveEvent;
            MeasurementSettingsRemoteObjectReceiveEvent -= OnMeasurementSettingsRemoteObjectReceiveEvent;
            DeviceConfigRemoteObjectReceiveEvent -= OnDeviceConfigRemoteObjectReceiveEvent;
            TriggerConfigRemoteObjectReceiveEvent -= OnTriggerConfigRemoteObjectEventReceiveEvent;
            m_SettingsReceived = false;
            if (m_pf300Pro == null && m_citrex == null) {
                Application.Current.Shutdown();
                Environment.Exit(0);
            }

            if (m_pf300Pro != null) {
                m_pf300Pro.ExceptionOccurred -= Device_ExceptionOccurred;
                m_pf300Pro.ValuesAllDataRemoteObject.AllDataHandler -= HandlePF300ProAllData;
                m_pf300Pro.ValuesFastDataRemoteObject.FastDataHandler -= HandlePF300ProFastData;
                m_pf300Pro.ValuesIrmaDataRemoteObject.IrmaDataHandler -= HandlePF300ProIrmaData;
                m_pf300Pro.MeasurementSettingsRemoteObject.PropertyChanged -= MeasurementSettingsRemoteObject_PropertyChanged;
                m_pf300Pro.DeviceSettingsRemoteObject.PropertyChanged -= DeviceSettingsRemoteObject_PropertyChanged;
                m_pf300Pro.TriggerSettingsRemoteObject.PropertyChanged -= TriggerSettingsRemoteObject_PropertyChanged;
                if (m_pf300Pro.IsConnected) {
                    m_pf300Pro.Disconnect();
                }
            }
            
            if (m_citrex == null) {
                Application.Current.Shutdown();
                Environment.Exit(0);
            }

            m_citrex.ExceptionOccurred -= Device_ExceptionOccurred;
            m_citrex.ValuesAllDataRemoteObject.AllDataHandler -= HandleCitrexAllData;
            m_citrex.ValuesFastDataRemoteObject.FastDataHandler -= HandleCitrexFastData;
            m_citrex.DeviceConfigRemoteObject.PropertyChanged -= OnDeviceConfigRemoteObject_PropertyChanged;
            m_citrex.TriggerConfigRemoteObject.PropertyChanged -= TriggerConfigRemoteObject_PropertyChanged;
            if (m_citrex.IsConnected) {
                m_citrex.Disconnect();
            }

            Application.Current.Shutdown();
            Environment.Exit(0);
        }

        /// <summary>
        /// Receive all PF300Pro all data values.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="eventData"></param>
        private void HandlePF300ProAllData(object sender, PF300ProAllData eventData) {
            if (eventData != null) {
                CheckSettingsReceived();
                m_content.Pf300ProSettings.PF300ProAllData = eventData;
            }
        }

        /// <summary>
        /// Receive all Citrex H5 all data values.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="allData"></param>
        private void HandleCitrexAllData(object sender, CitrexAllData allData) {
            if (allData != null) {
                CheckCitrexSettingsReceived();
                m_content.CitrexSettings.CitrexAllData = allData;
            }
        }

        /// <summary>
        /// Receive fast PF300Pro fast data values.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="eventData"></param>
        private void HandlePF300ProFastData(object sender, PF300ProFastData eventData) {
            if (eventData != null) {
                m_content.Pf300ProSettings.PF300ProFastData = eventData;
            }
        }

        /// <summary>
        /// Receive fast PF300Pro irma data values.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="eventData"></param>
        private void HandlePF300ProIrmaData(object sender, PF300ProIrmaData irmaData) {
            if (irmaData != null) {
                m_content.Pf300ProSettings.PF300ProIrmaData = irmaData;
            }
        }
        
        /// <summary>
        /// Receive fast Citrex H5 fast data values.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="fastData"></param>
        private void HandleCitrexFastData(object sender, CitrexFastData fastData) {
            if (fastData != null) {
                m_content.CitrexSettings.CitrexFastData = fastData;
            }
        }

        /// <summary>
        /// Catch unhandled application errors with a Shotdown.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="ex"></param>
        private static void CurrentDomain_UnhandledException(object sender, UnhandledExceptionEventArgs ex) {
            Debug.WriteLine((ex.ExceptionObject as Exception)?.Message);
            MessageBox.Show("Error: Lost connection: " + (ex.ExceptionObject as Exception)?.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error, MessageBoxResult.OK);
            Application.Current.Shutdown();
            Environment.Exit(0);
        }

        /// <summary>
        /// Catch device exceptions with device disconnection.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="ex"></param>
        private void Device_ExceptionOccurred(object? sender, Exception ex) {
            Debug.WriteLine(ex.Message);
            MessageBox.Show("Error: Lost connection: " + ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error, MessageBoxResult.OK);
            Disconnect(this, new RoutedEventArgs());
        }

        /// <summary>
        /// On change PF300Pro meassurement settings received.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="measurementSettingsRemoteObject"></param>
        private void OnMeasurementSettingsRemoteObjectReceiveEvent(object? sender, MeasurementSettingsRemoteObject measurementSettingsRemoteObject) {
            if (measurementSettingsRemoteObject != null &&
                m_activeSaving == false) {
                SetMeasurementSettingsRemoteObject(measurementSettingsRemoteObject);
            }
        }

        /// <summary>
        /// On change PF300Pro device settings received.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="deviceSettingsRemoteObject"></param>
        private void OnDeviceSettingsReceiveEvent(object? sender, DeviceSettingsRemoteObject deviceSettingsRemoteObject) {
            if (deviceSettingsRemoteObject != null &&
                m_activeSaving == false) {
                SetDeviceSettings(deviceSettingsRemoteObject);
            }
        }

        /// <summary>
        /// On change PF300Pro trigger settings received.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="triggerSettingsRemoteObject"></param>
        private void OnTriggerSettingsRemoteObjectReceiveEvent(object? sender, TriggerSettingsRemoteObject triggerSettingsRemoteObject) {
            if (triggerSettingsRemoteObject != null &&
                m_activeSaving == false) {
                SetTriggerSource(triggerSettingsRemoteObject);
            }
        }

        /// <summary>
        /// On change Citrex H5 device settings received.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="deviceConfigRemoteObjecte"></param>
        private void OnDeviceConfigRemoteObjectReceiveEvent(object? sender, DeviceConfigRemoteObject deviceConfigRemoteObjecte) {
            if (deviceConfigRemoteObjecte != null &&
                m_activeSaving == false) {
                SetDeviceConfigRemoteObject(deviceConfigRemoteObjecte);
            }
        }

        /// <summary>
        /// On change Citrex H5 trigger settings received.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="triggerConfigRemoteObject"></param>
        private void OnTriggerConfigRemoteObjectEventReceiveEvent(object? sender, TriggerConfigRemoteObject triggerConfigRemoteObject) {
            if (triggerConfigRemoteObject != null &&
                m_activeSaving == false) {
                SetCitrexTriggerSource(triggerConfigRemoteObject);
            }
        }

        private void SetMeasurementSettingsRemoteObject(MeasurementSettingsRemoteObject measurementSettingsRemoteObject) {
            m_content.Pf300ProSettings.MeasurementSettingsRemoteObject = measurementSettingsRemoteObject;
            Dispatcher.Invoke(() => {
                DeviceVentilationParameterPressureSource.SelectedValue = m_content.Pf300ProSettings.MeasurementSettingsRemoteObject.DeviceVentilationParameterPressureSource.ToString();
                DynamicPressureCompensationEnabled.IsChecked = m_content.Pf300ProSettings.MeasurementSettingsRemoteObject.DynamicPressureCompensationEnabled;
                DeviceFilterType.SelectedValue = m_content.Pf300ProSettings.MeasurementSettingsRemoteObject.DeviceFilterType.ToString();
                DeviceTriggerMode.SelectedValue = m_content.Pf300ProSettings.MeasurementSettingsRemoteObject.DeviceTriggerMode.ToString();
                DeviceGasType.SelectedValue = m_content.Pf300ProSettings.MeasurementSettingsRemoteObject.DeviceGasType.ToString();
                DeviceVolumeStandard.SelectedValue = m_content.Pf300ProSettings.MeasurementSettingsRemoteObject.DeviceVolumeStandard.ToString();

                DeviceGasTypeOxygenConcentration.Text = m_content.Pf300ProSettings.MeasurementSettingsRemoteObject.DeviceGasTypeOxygenConcentration.ToString();
                DeviceBaseFlowMode.SelectedValue = m_content.Pf300ProSettings.MeasurementSettingsRemoteObject.DeviceBaseFlowMode.ToString();
                DeviceBaseFlowValue.Text = m_content.Pf300ProSettings.MeasurementSettingsRemoteObject.DeviceBaseFlowValue.ToString();

                DeviceUlfVolumeStandard.SelectedValue = m_content.Pf300ProSettings.MeasurementSettingsRemoteObject.DeviceUlfVolumeStandard.ToString();
                DeviceUlfGasType.SelectedValue = m_content.Pf300ProSettings.MeasurementSettingsRemoteObject.DeviceUlfGasType.ToString();
            });
        }

        private void SetDeviceSettings(DeviceSettingsRemoteObject deviceSettingsRemoteObject) {
            m_content.Pf300ProSettings.DeviceSettingsRemoteObject = deviceSettingsRemoteObject;
            Dispatcher.Invoke(() => {
                DeviceRS232Protocol.SelectedValue = m_content.Pf300ProSettings.DeviceSettingsRemoteObject.DeviceRS232Protocol.ToString();
            });
        }

        private void SetDeviceConfigRemoteObject(DeviceConfigRemoteObject deviceConfigRemoteObjecte) {
            m_content.CitrexSettings.CitrexDeviceConfigRemoteObject = deviceConfigRemoteObjecte;
            Dispatcher.Invoke(() => {
                CitrexVentilationParameterPressureSource.SelectedValue = m_content.CitrexSettings.CitrexDeviceConfigRemoteObject.DeviceVentilationParameterPressureSource.ToString();
                CitrexVolumeStandard.SelectedValue = m_content.CitrexSettings.CitrexDeviceConfigRemoteObject.DeviceVolumeStandard.ToString();
                CitrexFilterType.SelectedValue = m_content.CitrexSettings.CitrexDeviceConfigRemoteObject.DeviceFilterType.ToString();
                CitrexGasType.SelectedValue = m_content.CitrexSettings.CitrexDeviceConfigRemoteObject.DeviceGasType.ToString();
                CitrexGasHumidity.Text = m_content.CitrexSettings.CitrexDeviceConfigRemoteObject.DeviceHumidity.ToString();
                CitrexPressureCompensationType.SelectedValue = m_content.CitrexSettings.CitrexDeviceConfigRemoteObject.DevicePressureCompensationType.ToString();
                CitrexRS232Protocol.SelectedValue = m_content.CitrexSettings.CitrexDeviceConfigRemoteObject.DeviceRS232Protocol.ToString();
                CitrexTriggerMode.SelectedValue = m_content.CitrexSettings.CitrexDeviceConfigRemoteObject.DeviceTriggerMode.ToString();
                //Only for H4
                //CitrexScaleXAxis.SelectedValue = m_content.CitrexSettings.CitrexDeviceConfigRemoteObject.DeviceScaleXAxis.ToString();
                CitrexAnalogOut1ValueModelId.SelectedValue = m_content.CitrexSettings.CitrexDeviceConfigRemoteObject.DeviceAnalogOut1ValueModelId.ToString();
                CitrexAnalogOut2ValueModelId.SelectedValue = m_content.CitrexSettings.CitrexDeviceConfigRemoteObject.DeviceAnalogOut2ValueModelId.ToString();        

                CitrexOwnerName.Text = m_content.CitrexSettings.CitrexDeviceConfigRemoteObject.DeviceOwnerName.ToString();
                CitrexCompanyName.Text = m_content.CitrexSettings.CitrexDeviceConfigRemoteObject.DeviceCompanyName.ToString();
                CitrexGasTypeOxygenConcentration.Text = m_content.CitrexSettings.CitrexDeviceConfigRemoteObject.DeviceGasTypeOxygenConcentration.ToString();
                CitrexBaseFlowValue.Text = m_content.CitrexSettings.CitrexDeviceConfigRemoteObject.DeviceBaseFlowValue.ToString();               
                CitrexScreenRotationLocked.IsChecked = m_content.CitrexSettings.CitrexDeviceConfigRemoteObject.DeviceScreenRotationLocked;
                CitrexBaseFlowEnabled.IsChecked = m_content.CitrexSettings.CitrexDeviceConfigRemoteObject.DeviceBaseFlowEnabled;
                CitrexOxygenOptionEnabled.IsChecked = m_content.CitrexSettings.CitrexDeviceConfigRemoteObject.DeviceOxygenOptionEnabled;                

                //Read Only settings
                CitrexMonitoringOptionEnabled.IsChecked = m_content.CitrexSettings.CitrexDeviceConfigRemoteObject.DeviceMonitoringOptionEnabled;
                CitrexTSI4000ProtocolOptionEnabled.IsChecked = m_content.CitrexSettings.CitrexDeviceConfigRemoteObject.DeviceTSI4000ProtocolOptionEnabled;
                CitrexFlowLabOptionEnabled.IsChecked = m_content.CitrexSettings.CitrexDeviceConfigRemoteObject.DeviceFlowLabOptionEnabled;
            });
        }

        private void SetTriggerSource(TriggerSettingsRemoteObject triggerSettingsRemoteObject) {
            m_content.Pf300ProSettings.TriggerSettingsRemoteObject = triggerSettingsRemoteObject;
            Dispatcher.Invoke(() => {
                if (m_content.Pf300ProSettings.TriggerSettingsRemoteObject?.DeviceAdultTrigger.Source != null &&
                    m_content.Pf300ProSettings.TriggerSettingsRemoteObject?.DevicePediatricTrigger != null &&
                    m_content.Pf300ProSettings.TriggerSettingsRemoteObject?.DevicePediatricTrigger.Source != null &&
                    m_content.Pf300ProSettings.TriggerSettingsRemoteObject?.DeviceHighFrequencyTrigger != null &&
                    m_content.Pf300ProSettings.TriggerSettingsRemoteObject?.DeviceHighFrequencyTrigger.Source != null) {
                    switch (m_content.Pf300ProSettings.MeasurementSettingsRemoteObject?.DeviceTriggerMode) {
                        case TriggerMode.Adult:
                            if (m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceAdultTrigger.Source != TriggerSource.Internal) {
                                m_content.Pf300ProSettings.IsVisibleTriggerSetting = Visibility.Hidden;
                            }
                            else {
                                m_content.Pf300ProSettings.IsVisibleTriggerSetting = Visibility.Visible;
                            }
                            DeviceTriggerSource.SelectedValue = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceAdultTrigger.Source.ToString();
                            DeviceTriggerStartSignal.SelectedValue = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceAdultTrigger.StartSignal.ToString();
                            DeviceTriggerStartDelay.Text = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceAdultTrigger.StartDelay.ToString();
                            DeviceTriggerStartEdge.SelectedValue = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceAdultTrigger.StartEdge.ToString();
                            DeviceTriggerStartLevel.Text = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceAdultTrigger.StartLevel.ToString();
                            DeviceTriggerEndSignal.SelectedValue = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceAdultTrigger.EndSignal.ToString();
                            DeviceTriggerEndDelay.Text = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceAdultTrigger.EndDelay.ToString();
                            DeviceTriggerEndEdge.SelectedValue = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceAdultTrigger.EndEdge.ToString();
                            DeviceTriggerEndLevel.Text = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceAdultTrigger.EndLevel.ToString();
                            break;
                        case TriggerMode.Pediatric:
                            if (m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DevicePediatricTrigger.Source != TriggerSource.Internal) {
                                m_content.Pf300ProSettings.IsVisibleTriggerSetting = Visibility.Hidden;
                            }
                            else {
                                m_content.Pf300ProSettings.IsVisibleTriggerSetting = Visibility.Visible;
                            }
                            DeviceTriggerSource.SelectedValue = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DevicePediatricTrigger.Source.ToString();
                            DeviceTriggerStartSignal.SelectedValue = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DevicePediatricTrigger.StartSignal.ToString();
                            DeviceTriggerStartDelay.Text = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DevicePediatricTrigger.StartDelay.ToString();
                            DeviceTriggerStartEdge.SelectedValue = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DevicePediatricTrigger.StartEdge.ToString();
                            DeviceTriggerStartLevel.Text = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DevicePediatricTrigger.StartLevel.ToString();
                            DeviceTriggerEndSignal.SelectedValue = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DevicePediatricTrigger.EndSignal.ToString();
                            DeviceTriggerEndDelay.Text = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DevicePediatricTrigger.EndDelay.ToString();
                            DeviceTriggerEndEdge.SelectedValue = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DevicePediatricTrigger.EndEdge.ToString();
                            DeviceTriggerEndLevel.Text = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DevicePediatricTrigger.EndLevel.ToString();
                            break;
                        case TriggerMode.HighFreq:
                            if (m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceHighFrequencyTrigger.Source != TriggerSource.Internal) {
                                m_content.Pf300ProSettings.IsVisibleTriggerSetting = Visibility.Hidden;
                            }
                            else {
                                m_content.Pf300ProSettings.IsVisibleTriggerSetting = Visibility.Visible;
                            }
                            DeviceTriggerSource.SelectedValue = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceHighFrequencyTrigger.Source.ToString();
                            DeviceTriggerStartSignal.SelectedValue = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceHighFrequencyTrigger.StartSignal.ToString();
                            DeviceTriggerStartDelay.Text = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceHighFrequencyTrigger.StartDelay.ToString();
                            DeviceTriggerStartEdge.SelectedValue = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceHighFrequencyTrigger.StartEdge.ToString();
                            DeviceTriggerStartLevel.Text = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceHighFrequencyTrigger.StartLevel.ToString();
                            DeviceTriggerEndSignal.SelectedValue = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceHighFrequencyTrigger.EndSignal.ToString();
                            DeviceTriggerEndDelay.Text = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceHighFrequencyTrigger.EndDelay.ToString();
                            DeviceTriggerEndEdge.SelectedValue = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceHighFrequencyTrigger.EndEdge.ToString();
                            DeviceTriggerEndLevel.Text = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceHighFrequencyTrigger.EndLevel.ToString();
                            break;
                        case TriggerMode.AutoTrigger:
                        default:
                            if (m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceAdultTrigger.Source != TriggerSource.Internal) {
                                m_content.Pf300ProSettings.IsVisibleTriggerSetting = Visibility.Hidden;
                            }
                            else {
                                m_content.Pf300ProSettings.IsVisibleTriggerSetting = Visibility.Visible;
                            }
                            DeviceTriggerSource.SelectedValue = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceAdultTrigger.Source.ToString();
                            DeviceTriggerStartSignal.SelectedValue = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceAdultTrigger.StartSignal.ToString();
                            DeviceTriggerStartDelay.Text = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceAdultTrigger.StartDelay.ToString();
                            DeviceTriggerStartEdge.SelectedValue = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceAdultTrigger.StartEdge.ToString();
                            DeviceTriggerStartLevel.Text = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceAdultTrigger.StartLevel.ToString();
                            DeviceTriggerEndSignal.SelectedValue = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceAdultTrigger.EndSignal.ToString();
                            DeviceTriggerEndDelay.Text = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceAdultTrigger.EndDelay.ToString();
                            DeviceTriggerEndEdge.SelectedValue = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceAdultTrigger.EndEdge.ToString();
                            DeviceTriggerEndLevel.Text = m_content.Pf300ProSettings.TriggerSettingsRemoteObject.DeviceAdultTrigger.EndLevel.ToString();
                            break;
                    }
                }
            });
        }

        private void SetCitrexTriggerSource(TriggerConfigRemoteObject triggerConfigRemoteObject) {
            m_content.CitrexSettings.CitrexTriggerConfigRemoteObject = triggerConfigRemoteObject;
            Dispatcher.Invoke(() => {
                if (m_content.CitrexSettings.CitrexDeviceConfigRemoteObject != null &&
                    m_content.CitrexSettings.CitrexDeviceConfigRemoteObject?.DeviceTriggerMode != null &&
                    m_content.CitrexSettings.CitrexTriggerConfigRemoteObject?.DeviceAdultTrigger.Source != null &&
                    m_content.CitrexSettings.CitrexTriggerConfigRemoteObject?.DevicePediatricTrigger != null &&
                    m_content.CitrexSettings.CitrexTriggerConfigRemoteObject?.DevicePediatricTrigger.Source != null &&
                    m_content.CitrexSettings.CitrexTriggerConfigRemoteObject?.DeviceHighFrequencyTrigger != null &&
                    m_content.CitrexSettings.CitrexTriggerConfigRemoteObject?.DeviceHighFrequencyTrigger.Source != null) {
                    switch (m_content.CitrexSettings.CitrexDeviceConfigRemoteObject.DeviceTriggerMode) {
                        case TriggerMode.Adult:
                            if (m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceAdultTrigger.Source != TriggerSource.Internal) {
                                m_content.CitrexSettings.IsVisibleCitrexTriggerSetting = Visibility.Hidden;
                            }
                            else {
                                m_content.CitrexSettings.IsVisibleCitrexTriggerSetting = Visibility.Visible;
                            }
                            CitrexTriggerSource.SelectedValue = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceAdultTrigger.Source.ToString();
                            CitrexTriggerStartSignal.SelectedValue = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceAdultTrigger.StartSignal.ToString();
                            CitrexTriggerStartDelay.Text = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceAdultTrigger.StartDelay.ToString();
                            CitrexTriggerStartEdge.SelectedValue = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceAdultTrigger.StartEdge.ToString();
                            CitrexTriggerStartLevel.Text = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceAdultTrigger.StartLevel.ToString();
                            CitrexTriggerEndSignal.SelectedValue = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceAdultTrigger.EndSignal.ToString();
                            CitrexTriggerEndDelay.Text = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceAdultTrigger.EndDelay.ToString();
                            CitrexTriggerEndEdge.SelectedValue = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceAdultTrigger.EndEdge.ToString();
                            CitrexTriggerEndLevel.Text = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceAdultTrigger.EndLevel.ToString();
                            break;
                        case TriggerMode.Pediatric:
                            if (m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DevicePediatricTrigger.Source != TriggerSource.Internal) {
                                m_content.CitrexSettings.IsVisibleCitrexTriggerSetting = Visibility.Hidden;
                            }
                            else {
                                m_content.CitrexSettings.IsVisibleCitrexTriggerSetting = Visibility.Visible;
                            }
                            CitrexTriggerSource.SelectedValue = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DevicePediatricTrigger.Source.ToString();
                            CitrexTriggerStartSignal.SelectedValue = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DevicePediatricTrigger.StartSignal.ToString();
                            CitrexTriggerStartDelay.Text = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DevicePediatricTrigger.StartDelay.ToString();
                            CitrexTriggerStartEdge.SelectedValue = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DevicePediatricTrigger.StartEdge.ToString();
                            CitrexTriggerStartLevel.Text = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DevicePediatricTrigger.StartLevel.ToString();
                            CitrexTriggerEndSignal.SelectedValue = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DevicePediatricTrigger.EndSignal.ToString();
                            CitrexTriggerEndDelay.Text = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DevicePediatricTrigger.EndDelay.ToString();
                            CitrexTriggerEndEdge.SelectedValue = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DevicePediatricTrigger.EndEdge.ToString();
                            CitrexTriggerEndLevel.Text = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DevicePediatricTrigger.EndLevel.ToString();
                            break;
                        case TriggerMode.HighFreq:
                            if (m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceHighFrequencyTrigger.Source != TriggerSource.Internal) {
                                m_content.CitrexSettings.IsVisibleCitrexTriggerSetting = Visibility.Hidden;
                            }
                            else {
                                m_content.CitrexSettings.IsVisibleCitrexTriggerSetting = Visibility.Visible;
                            }
                            CitrexTriggerSource.SelectedValue = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceHighFrequencyTrigger.Source.ToString();
                            CitrexTriggerStartSignal.SelectedValue = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceHighFrequencyTrigger.StartSignal.ToString();
                            CitrexTriggerStartDelay.Text = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceHighFrequencyTrigger.StartDelay.ToString();
                            CitrexTriggerStartEdge.SelectedValue = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceHighFrequencyTrigger.StartEdge.ToString();
                            CitrexTriggerStartLevel.Text = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceHighFrequencyTrigger.StartLevel.ToString();
                            CitrexTriggerEndSignal.SelectedValue = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceHighFrequencyTrigger.EndSignal.ToString();
                            CitrexTriggerEndDelay.Text = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceHighFrequencyTrigger.EndDelay.ToString();
                            CitrexTriggerEndEdge.SelectedValue = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceHighFrequencyTrigger.EndEdge.ToString();
                            CitrexTriggerEndLevel.Text = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceHighFrequencyTrigger.EndLevel.ToString();
                            break;
                        case TriggerMode.AutoTrigger:
                        default:
                            if (m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceAdultTrigger.Source != TriggerSource.Internal) {
                                m_content.CitrexSettings.IsVisibleCitrexTriggerSetting = Visibility.Hidden;
                            }
                            else {
                                m_content.CitrexSettings.IsVisibleCitrexTriggerSetting = Visibility.Visible;
                            }

                            CitrexTriggerSource.SelectedValue = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceAdultTrigger.Source.ToString();
                            CitrexTriggerStartSignal.SelectedValue = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceAdultTrigger.StartSignal.ToString();
                            CitrexTriggerStartDelay.Text = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceAdultTrigger.StartDelay.ToString();
                            CitrexTriggerStartEdge.SelectedValue = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceAdultTrigger.StartEdge.ToString();
                            CitrexTriggerStartLevel.Text = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceAdultTrigger.StartLevel.ToString();
                            CitrexTriggerEndSignal.SelectedValue = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceAdultTrigger.EndSignal.ToString();
                            CitrexTriggerEndDelay.Text = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceAdultTrigger.EndDelay.ToString();
                            CitrexTriggerEndEdge.SelectedValue = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceAdultTrigger.EndEdge.ToString();
                            CitrexTriggerEndLevel.Text = m_content.CitrexSettings.CitrexTriggerConfigRemoteObject.DeviceAdultTrigger.EndLevel.ToString();
                            break;
                    }
                }
            });
        }

        /// <summary>
        /// Receive changed Citrex H5 tigger settings.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TriggerConfigRemoteObject_PropertyChanged(object? sender, PropertyChangedEventArgs e) {
            if (m_activeSaving == false && sender != null) {
                CheckCitrexSettingsReceived();
                SetCitrexTriggerSource((TriggerConfigRemoteObject)sender);
            }
        }

        /// <summary>
        /// Receive changed Citrex H5 device settings.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnDeviceConfigRemoteObject_PropertyChanged(object? sender, PropertyChangedEventArgs e) {
            if (m_activeSaving == false && sender != null) {
                CheckCitrexSettingsReceived();
                SetDeviceConfigRemoteObject((DeviceConfigRemoteObject)sender);
            }
        }

        /// <summary>
        /// Receive changed PF300Pro meassurement settings.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MeasurementSettingsRemoteObject_PropertyChanged(object? sender, PropertyChangedEventArgs e) {
            if (m_activeSaving == false && sender != null) {
                CheckSettingsReceived();
                SetMeasurementSettingsRemoteObject((MeasurementSettingsRemoteObject)sender);
            }
        }

        /// <summary>
        /// Receive changed PF300Pro device settings.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DeviceSettingsRemoteObject_PropertyChanged(object? sender, PropertyChangedEventArgs e) {
            if (m_activeSaving == false && sender != null) {
                CheckSettingsReceived();
                SetDeviceSettings((DeviceSettingsRemoteObject)sender);
            }
        }

        /// <summary>
        /// Receive changed PF300Pro trigger settings.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TriggerSettingsRemoteObject_PropertyChanged(object? sender, PropertyChangedEventArgs e) {
            if (m_activeSaving == false && sender != null) {
                CheckSettingsReceived();
                SetTriggerSource((TriggerSettingsRemoteObject)sender);
            }
        }

        /// <summary>
        /// Check if Citrex H5 settings are allready received after connection.
        /// </summary>
        private void CheckCitrexSettingsReceived() {
            if (m_citrex == null) { 
                return;
            }

            var deviceConfigRemoteObject = m_citrex.DeviceConfigRemoteObject;
            if (m_SettingsReceived == false) {
                if (deviceConfigRemoteObject.SettingsReceived) {
                    m_SettingsReceived = true;
                    if (DeviceConfigRemoteObjectReceiveEvent != null) {
                        DeviceConfigRemoteObjectReceiveEvent.Invoke(this, deviceConfigRemoteObject);
                    }
                } else {
                    deviceConfigRemoteObject.RequestSettings();
                }

                var triggerConfigRemoteObject = m_citrex.TriggerConfigRemoteObject;
                if (triggerConfigRemoteObject.SettingsReceived) {
                    m_SettingsReceived = true;
                    if (TriggerConfigRemoteObjectReceiveEvent != null) {
                        TriggerConfigRemoteObjectReceiveEvent.Invoke(this, triggerConfigRemoteObject);
                    } 
                } else {
                    triggerConfigRemoteObject.RequestSettings();
                }
            }
        }

        /// <summary>
        /// Check if PF300Pro settings are allready received after connection.
        /// </summary>
        private void CheckSettingsReceived() {
            if(m_pf300Pro == null) {
                return;
            }
            var settingRequestRemoteObject = m_pf300Pro.SettingRequestRemoteObject;
            settingRequestRemoteObject.RequestDeviceSettings();
            settingRequestRemoteObject.RequestMeasurementSettings();
            var deviceConfigRemoteObject = m_pf300Pro.DeviceSettingsRemoteObject;
            if (m_SettingsReceived == false) {
                if (deviceConfigRemoteObject.SettingsReceived) {
                    m_SettingsReceived = true;
                    if (DeviceSettingsReceivedReceiveEvent != null) {
                        DeviceSettingsReceivedReceiveEvent.Invoke(this, deviceConfigRemoteObject);
                    }
                }
            }
            if (m_SettingsReceived == false) {
                var triggerSettingsRemoteObject = m_pf300Pro.TriggerSettingsRemoteObject;
                if (triggerSettingsRemoteObject.SettingsReceived) {
                    m_SettingsReceived = true;
                    if (TriggerSettingsRemoteObjectReceiveEvent != null) {
                        TriggerSettingsRemoteObjectReceiveEvent.Invoke(this, triggerSettingsRemoteObject);
                    }
                }
            }
            if (m_SettingsReceived == false) {
                var measurementSettingsRemoteObject = m_pf300Pro.MeasurementSettingsRemoteObject;
                if (measurementSettingsRemoteObject.SettingsReceived) {
                    m_SettingsReceived = true;
                    if (MeasurementSettingsRemoteObjectReceiveEvent != null) {
                        MeasurementSettingsRemoteObjectReceiveEvent.Invoke(this, measurementSettingsRemoteObject);
                    }
                }
            }
        }

        /// <summary>
        /// Sett PF300Pro device settings after loading them.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SetDeviceSettings(object sender, RoutedEventArgs e) {
            m_activeSaving = true;
            Dispatcher.Invoke(() => {
                foreach (Pf300ProRS232Protocol pf300ProRS232Protocol in Enum.GetValues(typeof(Pf300ProRS232Protocol))) {
                    if (pf300ProRS232Protocol.ToString() == DeviceRS232Protocol.SelectedValue.ToString()) {
                        m_activeSaving = false;
                        m_content.Pf300ProSettings.DeviceSettingsRemoteObject?.SendRS232Protocol(pf300ProRS232Protocol);
                        break;
                    }
                }

            });
        }

        /// <summary>
        /// Sett PF300Pro meassurement settings after loading them.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SetMeasurement(object sender, RoutedEventArgs e) {
            m_activeSaving = true;
            Dispatcher.Invoke(() => {
                foreach (VolumeStandard volumeStandard in Enum.GetValues(typeof(VolumeStandard))) {
                    if (volumeStandard.ToString() == DeviceVolumeStandard.SelectedValue.ToString()) {
                        m_content.Pf300ProSettings.MeasurementSettingsRemoteObject?.SendVolumeStandard(volumeStandard);
                        break;
                    }
                }
                foreach (Pf300ProGasType gasType in Enum.GetValues(typeof(Pf300ProGasType))) {
                    if (gasType.ToString() == DeviceGasType.SelectedValue.ToString()) {
                        m_content.Pf300ProSettings.MeasurementSettingsRemoteObject?.SendGasType(gasType);
                        break;
                    }
                }
                foreach (UlfVolumeStandard ulfVolumeStandard in Enum.GetValues(typeof(UlfVolumeStandard))) {
                    if (ulfVolumeStandard.ToString() == DeviceUlfVolumeStandard.SelectedValue.ToString()) {
                        m_content.Pf300ProSettings.MeasurementSettingsRemoteObject?.SendUlfVolumeStandard(ulfVolumeStandard);
                        break;
                    }
                }
                foreach (UlfGasType ulfGasType in Enum.GetValues(typeof(UlfGasType))) {
                    if (ulfGasType.ToString() == DeviceUlfGasType.SelectedValue.ToString()) {
                        m_content.Pf300ProSettings.MeasurementSettingsRemoteObject?.SendUlfGasType(ulfGasType);
                        break;
                    }
                }
                foreach (FilterType filterType in Enum.GetValues(typeof(FilterType))) {
                    if (filterType.ToString() == DeviceFilterType.SelectedValue.ToString()) {
                        m_content.Pf300ProSettings.MeasurementSettingsRemoteObject?.SendFilterType(filterType);
                        break;
                    }
                }
                foreach (Pf300ProVentilationParameterPressureSource respiratoryPressureSource in Enum.GetValues(typeof(Pf300ProVentilationParameterPressureSource))) {
                    if (respiratoryPressureSource.ToString() == DeviceVentilationParameterPressureSource.SelectedValue.ToString()) {
                        m_content.Pf300ProSettings.MeasurementSettingsRemoteObject?.SendVentilationParameterPressureSource(respiratoryPressureSource);
                        break;
                    }
                }
                foreach (BaseFlowMode baseFlowMode in Enum.GetValues(typeof(BaseFlowMode))) {
                    if (baseFlowMode.ToString() == DeviceBaseFlowMode.SelectedValue.ToString()) {
                        m_content.Pf300ProSettings.MeasurementSettingsRemoteObject?.SendBaseFlowEnabled(baseFlowMode);
                        break;
                    }
                }
                
                m_content.Pf300ProSettings.MeasurementSettingsRemoteObject?.SendBaseFlowValue(float.Parse(DeviceBaseFlowValue.Text));
                m_content.Pf300ProSettings.MeasurementSettingsRemoteObject?.SendGasTypeOxygenConcentration(float.Parse(DeviceGasTypeOxygenConcentration.Text));
                m_activeSaving = false;
                m_content.Pf300ProSettings.MeasurementSettingsRemoteObject?.SendDynamicPressureCompensationEnabled(DynamicPressureCompensationEnabled.IsChecked == true);
            });
        }

        /// <summary>
        /// Sett PF300Pro trigger settings after loading them.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SetTrigger(object sender, RoutedEventArgs e) {
            if(m_pf300Pro == null) {
                return;
            }
            m_activeSaving = true;
            Dispatcher.Invoke(() => {
                foreach (TriggerMode triggerMode in Enum.GetValues(typeof(TriggerMode))) {
                    if (triggerMode.ToString() == DeviceTriggerMode.SelectedValue.ToString()) {
                        m_content.Pf300ProSettings.MeasurementSettingsRemoteObject?.SendTriggerMode(triggerMode);
                    }
                }
                var triggerSource = TriggerSource.External;
                foreach (TriggerSource thisTriggerSource in Enum.GetValues(typeof(TriggerSource))) {
                    if (thisTriggerSource.ToString() == DeviceTriggerSource.SelectedValue.ToString()) {
                        triggerSource = thisTriggerSource;
                    }
                }
                var startSignal = TriggerSignal.Flow;
                var endSignal = TriggerSignal.Flow;
                foreach (TriggerSignal triggerSignal in Enum.GetValues(typeof(TriggerSignal))) {
                    if (triggerSignal.ToString() == DeviceTriggerStartSignal.SelectedValue.ToString()) {
                        startSignal = triggerSignal;
                    }
                    if (triggerSignal.ToString() == DeviceTriggerEndSignal.SelectedValue.ToString()) {
                        endSignal = triggerSignal;
                    }
                }
                var startEdge = TriggerEdge.Rising;
                var endEdge = TriggerEdge.Falling;
                foreach (TriggerEdge triggerEdge in Enum.GetValues(typeof(TriggerEdge))) {
                    if (triggerEdge.ToString() == DeviceTriggerStartEdge.SelectedValue.ToString()) {
                        startEdge = triggerEdge;
                    }
                    if (triggerEdge.ToString() == DeviceTriggerEndEdge.SelectedValue.ToString()) {
                        endEdge = triggerEdge;
                    }
                }

                TriggerConfig trigger = new TriggerConfig(triggerSource,
                    startSignal,
                    startEdge,
                    float.Parse(DeviceTriggerStartLevel.Text),
                    UInt16.Parse(DeviceTriggerStartDelay.Text),
                    endSignal,
                    endEdge,
                    float.Parse(DeviceTriggerEndLevel.Text),
                    UInt16.Parse(DeviceTriggerEndDelay.Text));

                m_activeSaving = false;
                var triggerModes = TriggerMode.Adult.ToString();
                if (DeviceTriggerMode.SelectedValue != null) {
                    triggerModes = DeviceTriggerMode.SelectedValue.ToString();
                }
                if(triggerModes == null) {
                    triggerModes = TriggerMode.Adult.ToString();
                }
                switch (Enum.Parse(typeof(TriggerMode), triggerModes)) {
                    case TriggerMode.Adult:
                        m_pf300Pro.TriggerSettingsRemoteObject.SendTriggerConfigAdult(trigger);
                        break;
                    case TriggerMode.Pediatric:
                        m_pf300Pro.TriggerSettingsRemoteObject.SendTriggerConfigPediatric(trigger);
                        break;
                    case TriggerMode.HighFreq:
                        m_pf300Pro.TriggerSettingsRemoteObject.SendTriggerConfigHighFrequency(trigger);
                        break;
                    default:
                        break;
                }
 
            });
        }

        /// <summary>
        /// Check if Citrex H5 trigger settings after loading them.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SetCitrexTrigger(object sender, RoutedEventArgs e) {
            if(m_citrex == null) {
                return;
            }

            m_activeSaving = true;
            Dispatcher.Invoke(() => {
                foreach (TriggerMode triggerMode in Enum.GetValues(typeof(TriggerMode))) {
                    if (triggerMode.ToString() == CitrexTriggerMode.SelectedValue.ToString()) {
                        m_content.CitrexSettings.CitrexDeviceConfigRemoteObject?.SendTriggerMode(triggerMode);
                    }
                }
                var triggerSource = TriggerSource.External;
                foreach (TriggerSource thisTriggerSource in Enum.GetValues(typeof(TriggerSource))) {
                    if (thisTriggerSource.ToString() == CitrexTriggerSource.SelectedValue.ToString()) {
                        triggerSource = thisTriggerSource;
                    }
                }
                var startSignal = TriggerSignal.Flow;
                var endSignal = TriggerSignal.Flow;
                foreach (TriggerSignal triggerSignal in Enum.GetValues(typeof(TriggerSignal))) {
                    if (triggerSignal.ToString() == CitrexTriggerStartSignal.SelectedValue.ToString()) {
                        startSignal = triggerSignal;
                    }
                    if (triggerSignal.ToString() == CitrexTriggerEndSignal.SelectedValue.ToString()) {
                        endSignal = triggerSignal;
                    }
                }
                var startEdge = TriggerEdge.Rising;
                var endEdge = TriggerEdge.Falling;
                foreach (TriggerEdge triggerEdge in Enum.GetValues(typeof(TriggerEdge))) {
                    if (triggerEdge.ToString() == CitrexTriggerStartEdge.SelectedValue.ToString()) {
                        startEdge = triggerEdge;
                    }
                    if (triggerEdge.ToString() == CitrexTriggerEndEdge.SelectedValue.ToString()) {
                        endEdge = triggerEdge;
                    }
                }

                TriggerConfig trigger = new TriggerConfig(triggerSource,
                    startSignal,
                    startEdge,
                    float.Parse(CitrexTriggerStartLevel.Text),
                    UInt16.Parse(CitrexTriggerStartDelay.Text),
                    endSignal,
                    endEdge,
                    float.Parse(CitrexTriggerEndLevel.Text),
                    UInt16.Parse(CitrexTriggerEndDelay.Text));

                m_activeSaving = false;

                var triggerModes = CitrexTriggerMode.SelectedValue.ToString();
                if (CitrexTriggerMode.SelectedValue == null) {
                    triggerModes = TriggerMode.Adult.ToString();
                }
                if (triggerModes == null) {
                    triggerModes = TriggerMode.Adult.ToString();
                }
                switch (Enum.Parse(typeof(TriggerMode), triggerModes)) {
                    case TriggerMode.Adult:
                        m_citrex.TriggerConfigRemoteObject.SendTriggerConfigAdult(trigger);
                        break;
                    case TriggerMode.Pediatric:
                        m_citrex.TriggerConfigRemoteObject.SendTriggerConfigPediatric(trigger);
                        break;
                    case TriggerMode.HighFreq:
                        m_citrex.TriggerConfigRemoteObject.SendTriggerConfigHighFrequency(trigger);
                        break;
                    default:
                        break;
                }
     
            });
        }

        /// <summary>
        /// Check if Citrex H5 device settings after loading them.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SetCitrexMeasurements(object sender, RoutedEventArgs e) {
            m_activeSaving = true;
            try {
                Dispatcher.Invoke(() => {
                    foreach (VolumeStandard volumeStandard in Enum.GetValues(typeof(VolumeStandard))) {
                        if (volumeStandard.ToString() == CitrexVolumeStandard.SelectedValue.ToString()) {
                            m_content.CitrexSettings.CitrexDeviceConfigRemoteObject?.SendVolumeStandard(volumeStandard);
                        }
                    }
                    foreach (CitrexGasType gasType in Enum.GetValues(typeof(CitrexGasType))) {
                        if (gasType.ToString() == CitrexGasType.SelectedValue.ToString()) {
                            m_content.CitrexSettings.CitrexDeviceConfigRemoteObject?.SendGasType(gasType);
                        }
                    }
                    foreach (FilterType filterType in Enum.GetValues(typeof(FilterType))) {
                        if (filterType.ToString() == CitrexFilterType.SelectedValue.ToString()) {
                            m_content.CitrexSettings.CitrexDeviceConfigRemoteObject?.SendFilterType(filterType);
                        }
                    }
                    m_content.CitrexSettings.CitrexDeviceConfigRemoteObject?.SendHumidity(float.Parse(CitrexGasHumidity.Text));
                    foreach (SensorPressureCompensationType sensorPressureCompensationType in Enum.GetValues(typeof(SensorPressureCompensationType))) {
                        if (sensorPressureCompensationType.ToString() == CitrexPressureCompensationType.SelectedValue.ToString()) {
                            m_content.CitrexSettings.CitrexDeviceConfigRemoteObject?.SendPressureCompensationType(sensorPressureCompensationType);
                        }
                    }
                    foreach (CitrexVentilationParameterPressureSource respiratoryPressureSource in Enum.GetValues(typeof(CitrexVentilationParameterPressureSource))) {
                        if (respiratoryPressureSource.ToString() == CitrexVentilationParameterPressureSource.SelectedValue.ToString()) {
                            m_activeSaving = false;
                            m_content.CitrexSettings.CitrexDeviceConfigRemoteObject?.SendVentilationParameterPressureSource(respiratoryPressureSource);
                        }
                    }
                });
            }
            catch (Exception ex) {
                Debug.WriteLine(ex.Message);
            }
        }

        /// <summary>
        /// Check if Citrex H5 device settings after loading them.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SetCitrexDeviceConfigs(object sender, RoutedEventArgs e) {
            m_activeSaving = true;
            try {
                Dispatcher.Invoke(() => {
                    foreach (AnalogOutValueModelId analogOutValueModelId in Enum.GetValues(typeof(AnalogOutValueModelId))) {
                        if (analogOutValueModelId.ToString() == CitrexAnalogOut1ValueModelId.SelectedValue.ToString()) {
                            m_content.CitrexSettings.CitrexDeviceConfigRemoteObject?.SendAnalogOut1ValueModelId(analogOutValueModelId);
                        }
                        if (analogOutValueModelId.ToString() == CitrexAnalogOut2ValueModelId.SelectedValue.ToString()) {
                            m_content.CitrexSettings.CitrexDeviceConfigRemoteObject?.SendAnalogOut2ValueModelId(analogOutValueModelId);
                        }
                    }

                    foreach (CitrexRS232Protocol citrexRS232Protocol in Enum.GetValues(typeof(CitrexRS232Protocol))) {
                        if (citrexRS232Protocol.ToString() == CitrexRS232Protocol.SelectedValue.ToString()) {
                            m_content.CitrexSettings.CitrexDeviceConfigRemoteObject?.SendRS232Protocol(citrexRS232Protocol);
                        }
                    }

                    m_content.CitrexSettings.CitrexDeviceConfigRemoteObject?.SendOwnerName(CitrexOwnerName.Text);
                    m_content.CitrexSettings.CitrexDeviceConfigRemoteObject?.SendOwnerCompany(CitrexCompanyName.Text);

                    m_content.CitrexSettings.CitrexDeviceConfigRemoteObject?.SendBaseFlowEnabled(CitrexBaseFlowEnabled.IsChecked == true);
                    m_content.CitrexSettings.CitrexDeviceConfigRemoteObject?.SendBaseFlowValue(float.Parse(CitrexBaseFlowValue.Text));

                    m_content.CitrexSettings.CitrexDeviceConfigRemoteObject?.SendScreenRotationLocked(CitrexScreenRotationLocked.IsChecked == true);
                    float o2Concentration = 0;
                    if (CitrexOxygenOptionEnabled.IsChecked == true) {
                        o2Concentration = float.Parse(CitrexGasTypeOxygenConcentration.Text);
                    }
                    m_content.CitrexSettings.CitrexDeviceConfigRemoteObject?.SendGasTypeOxygenConcentration(o2Concentration);
                });
            }
            catch(Exception ex) {
                Debug.WriteLine(ex.Message);
            }
        }

        /// <summary>
        /// Connect to the available Citrex H5 devices.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ConnectCitrex(object sender, RoutedEventArgs e) {
            m_citrex = CitrexConnection.Connect();
            m_citrex.ValuesAllDataRemoteObject.AllDataHandler += HandleCitrexAllData;
            m_citrex.ValuesFastDataRemoteObject.FastDataHandler += HandleCitrexFastData;
            m_citrex.ExceptionOccurred += Device_ExceptionOccurred;
            m_citrex.DeviceConfigRemoteObject.PropertyChanged += OnDeviceConfigRemoteObject_PropertyChanged;
            m_citrex.TriggerConfigRemoteObject.PropertyChanged += TriggerConfigRemoteObject_PropertyChanged;
        }

        /// <summary>
        /// Connect to the available PF300Pro devices.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Connect(object sender, RoutedEventArgs e) {
            m_pf300Pro = Pf300ProConnection.Connect();
            m_pf300Pro.ExceptionOccurred += Device_ExceptionOccurred;
            m_pf300Pro.ValuesAllDataRemoteObject.AllDataHandler += HandlePF300ProAllData;
            m_pf300Pro.ValuesFastDataRemoteObject.FastDataHandler += HandlePF300ProFastData;
            m_pf300Pro.ValuesIrmaDataRemoteObject.IrmaDataHandler += HandlePF300ProIrmaData;
            m_pf300Pro.MeasurementSettingsRemoteObject.PropertyChanged += MeasurementSettingsRemoteObject_PropertyChanged;
            m_pf300Pro.DeviceSettingsRemoteObject.PropertyChanged += DeviceSettingsRemoteObject_PropertyChanged;
            m_pf300Pro.TriggerSettingsRemoteObject.PropertyChanged += TriggerSettingsRemoteObject_PropertyChanged;
        }

        /// <summary>
        /// Disconnect all active connections.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Disconnect(object sender, RoutedEventArgs e) {
            m_SettingsReceived = false;
            Pf300ProConnection.Disconnect();
            CitrexConnection.Disconnect();
        }

        /// <summary>
        /// Set visability of the PF300Pro trigger source fields.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DeviceTriggerSource_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e) {
            if (DeviceTriggerSource.SelectedItem.ToString() != "Internal") {
                m_content.Pf300ProSettings.IsVisibleTriggerSetting = Visibility.Hidden;
            }
            else {
                m_content.Pf300ProSettings.IsVisibleTriggerSetting = Visibility.Visible;
            }
        }

        /// <summary>
        /// Set visability of the Citrex H5 trigger source fields.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CitrexTriggerSource_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e) {
            if (CitrexTriggerSource.SelectedItem.ToString() != "Internal") {
                m_content.CitrexSettings.IsVisibleCitrexTriggerSetting = Visibility.Hidden;
            }
            else {
                m_content.CitrexSettings.IsVisibleCitrexTriggerSetting = Visibility.Visible;
            }
        }

        FlowAnalyserPro? m_pf300Pro;
        Citrex? m_citrex;

        private bool m_SettingsReceived;
        private DeviceSettings m_content;
        private bool m_activeSaving;
    }
}
